/*
 * InvocationContextImpl.java
 */

package com.example.interceptor;

import java.lang.reflect.Method;
import java.security.Principal;
import java.util.HashMap;
import java.util.Map;
import javax.ejb.EJBContext;
import org.aopalliance.intercept.MethodInvocation;
import com.example.util.ContextUtil;

/**
 * 
 */
public class InvocationContextImpl implements InvocationContext {

    /**
     * invocation parameters.
     */
    private Object[] parameters;

    /**
     * MethodInvocation
     */
    private MethodInvocation methodInvocation;

    /**
     * Context data
     */
    private HashMap contextData = new HashMap();

    /**
     * Construct new InvocationContext instance.
     * @param inv MethodInvocation
     */
    public InvocationContextImpl(MethodInvocation inv) {
        if (inv == null) {
            throw new IllegalArgumentException(
                    "Null MethodInvocation is supplied.");
        }

        this.methodInvocation = inv;
        this.parameters = methodInvocation.getArguments();
    }

    /**
     * Obtains the target object.
     * @return target object.
     */
    public Object getBean() {
        return methodInvocation.getThis();
    }

    /**
     * Obtains the target method.
     * @return target method.
     */
    public Method getMethod() {
        return methodInvocation.getMethod();
    }

    /**
     * Obtains the parameter list of the target method.
     * @return parameter list.
     */
    public Object[] getParameters() {
        return parameters;
    }

    /**
     * Sets the modified parameter list before method invocation.
     * @param args method parameter list.
     */
    public void setParameters(Object[] args) {
        if (args == null
            || parameters.length != args.length) {
            throw new IllegalArgumentException();
        }
        for (int i = 0; i < args.length; i++) {
            parameters[i] = args[i];
        }
    }

    /**
     * Obtains EJB context.
     * @return EJB context.
     */
    public EJBContext getEJBContext() {
        return ContextUtil.getEJBContext();
    }

    /**
     * Obtains context data.
     * @return context data.
     */
    public Map getContextData() {
        return contextData;
    }

    /**
     * Invokes the target method.
     * @return the result of the method invocation.
     * @throws Exception thrown if method invocation failed.
     */
    public Object proceed() throws Exception {
        try {
            return methodInvocation.proceed();
        }
        catch (Throwable th) {
            if (th instanceof Error) {
                throw (Error) th;
            }
            else {
                throw (Exception) th;
            }
        }
    }
}
