/*
 * MethodGenerator.java
 */

package com.example.tool;

import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.MethodDoc;
import com.sun.javadoc.Parameter;
import com.sun.javadoc.Tag;
import java.io.IOException;
import java.lang.reflect.Method;
import com.example.util.ResourceUtil;

/**
 * Generate source code fragments of the business method.
 */
class MethodGenerator {
    
    /** MethodDoc */
    private MethodDoc methoddoc;
    /** Source code encoding */
    private static final String ENCODING = "UTF-8";
    /** Class template file name */
    private static final String TEMPLATE_NAME =
            "CreateSessionBean_method.template";
    /** Comment head. */
    private static final String COMMENT_HEAD = "\n     * ";
    /** Method template */
    private static String template;
    
    /** Creates a new instance of MethodGenerator */
    public MethodGenerator(MethodDoc methoddoc) throws IOException {
        this.methoddoc = methoddoc;
        if (template == null) {
            template = ResourceUtil.getResourceAsString(this.getClass(),
                    TEMPLATE_NAME, ENCODING);
        }
    }
    
    /** Generate method source. */
    public String generate() {
        String methodComment = methoddoc.commentText();
        String tags = generateTags() + convertAnnotationsToTags();
        String returnType = methoddoc.returnType().toString();
        String methodName = methoddoc.name();
        String paramSpec = generateParamSpec();
        String throwSpec = generateThrowSpec();
        String returnStat = returnType.equals("void") ? "" : "return ";
        String paramCall = generateParamCall();
        
        String gensrc = template;
        gensrc = gensrc.replaceAll("%METHOD_COMMENT%",  methodComment);
        gensrc = gensrc.replaceAll("%TAGS%",  tags);
        gensrc = gensrc.replaceAll("%RETURN_TYPE%", returnType);
        gensrc = gensrc.replaceAll("%METHOD_NAME%", methodName);
        gensrc = gensrc.replaceAll("%PARAM_SPEC%", paramSpec);
        gensrc = gensrc.replaceAll("%THROW_SPEC%", throwSpec);
        gensrc = gensrc.replaceAll("%RETURN_STAT%", returnStat);
        gensrc = gensrc.replaceAll("%PARAM_CALL%", paramCall);
        return gensrc;
    }
    
    /** Generate tags. */
    private String generateTags() {
        Tag tags[] = methoddoc.tags();
        
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < tags.length;) {
            buf.append(tags[i].kind()).append(" ");
            if (!tags[i].kind().equals(tags[i].name())) {
                buf.append(tags[i].name()).append(" ");
            }
            buf.append(tags[i].text());
            if (++i != tags.length) {
                buf.append(COMMENT_HEAD);
            }
        }
        return buf.toString();
    }
    
    /** Convert annotations to XDoclet tags. */
    private String convertAnnotationsToTags() {
        Class c = null;
        try {
            c = Class.forName("com.example.tool.AnnotationUtil");
        }
        catch (ClassNotFoundException e) {
            return "";
        }
        try {
            // AnnotationUtil.convertToTags(methoddoc)を呼び出します．
            Method m = c.getMethod("convertToTags",
                    new Class[]{MethodDoc.class});
            return (String)m.invoke(null, new Object[]{methoddoc});
        }
        catch (Exception e) {
            throw new IllegalStateException(e.toString());
        }
    }
    
    /** Generate method parameter specifications. */
    private String generateParamSpec() {
        Parameter params[] = methoddoc.parameters();
        
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < params.length;) {
            buf.append(params[i].toString());
            if (++i != params.length) {
                buf.append(", ");
            }
        }
        return buf.toString();
    }
    
    /** Generate method call parameters. */
    private String generateParamCall() {
        Parameter params[] = methoddoc.parameters();
        
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < params.length;) {
            buf.append(params[i].name());
            if (++i != params.length) {
                buf.append(", ");
            }
        }
        return buf.toString();
    }
    
    /** Generate throws clause. */
    private String generateThrowSpec() {
        ClassDoc throwns[] = methoddoc.thrownExceptions();
        
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < throwns.length;) {
            if (i == 0) {
                buf.append("throws ");
            }
            buf.append(throwns[i].toString());
            if (++i != throwns.length) {
                buf.append(", ");
            }
        }
        return buf.toString();
    }
}
